@@ -59,6 +59,12 @@ def var_token(m):
     return end, 'var', value
 
 
+def rvalue_token(m):
+    """ Produces variable token as r-value expression.
+    """
+    return m.end(), 'var', str(m.group(1).strip())
+
+
 # region: parser config
 
 def configure_parser(parser):
@@ -101,7 +107,7 @@ def parse_var(value):
     if '!!' not in value:
         return value, None
     var, var_filter = value.rsplit('!!', 1)
-    return var, var_filter.split('!')
+    return var, var_filter.strip().split('!')
 
 
 # region: block_builders
@@ -115,7 +121,7 @@ def build_extends(builder, lineno, token, nodes):
         return False
     extends, nodes = value
     for lineno, token, value in nodes:
-        if token in ('def ', 'require'):
+        if token in ('def ', 'require', 'import ', 'from '):
             builder.build_token(lineno, token, value)
     lineno = builder.lineno
     builder.add(lineno + 1, 'return _r(' + extends +
@@ -134,14 +140,14 @@ def build_module(builder, lineno, token, nodes):
 def build_import(builder, lineno, token, value):
     assert token == 'import '
     name, var = value
-    builder.add(lineno, var + ' = ' + '_i(' + name + ')')
+    builder.add(lineno, var + ' = _i(' + name + ')')
     return True
 
 
 def build_from(builder, lineno, token, value):
     assert token == 'from '
     name, var, alias = value
-    builder.add(lineno, alias + ' = ' + '_i(' + name
+    builder.add(lineno, alias + ' = _i(' + name
                 + ').local_defs[\'' + var + '\']')
     return True
 
@@ -151,13 +157,19 @@ def build_render_single_markup(builder, lineno, token, nodes):
     assert token == 'render'
     if len(nodes) > 1:
         return False
+    if len(nodes) == 0:
+        builder.add(lineno, "return ''")
+        return True
     ln, token, nodes = nodes[0]
     if token != 'out' or len(nodes) > 1:
         return False
     ln, token, value = nodes[0]
     if token != 'markup':
         return False
-    builder.add(ln, "return " + value)
+    if value:
+        builder.add(ln, 'return ' + value)
+    else:
+        builder.add(ln, "return ''")
     return True
 
 
@@ -181,7 +193,7 @@ def build_def_syntax_check(builder, lineno, token, value):
         token = token.rstrip()
         error = """\
 The compound statement '%s' is not allowed here. \
-Add a line before it with @#ignore.
+Add a line before it with @#ignore or leave it empty.
 
 %s
     @#ignore
@@ -259,8 +271,8 @@ def build_out(builder, lineno, token, nodes):
     assert token == 'out'
     for lineno, token, value in nodes:
         if token == 'include':
-            builder.add(lineno, 'w(' + '_r(' + value +
-                        ', ctx, local_defs, super_defs)' + ')')
+            builder.add(lineno, 'w(_r(' + value +
+                        ', ctx, local_defs, super_defs))')
         elif token == 'var':
             var, var_filters = value
             if var_filters:
@@ -319,6 +331,8 @@ class CoreExtension(object):
                   stmt_token),
             200: (re.compile(r'%s(\w+(\.\w+)*)' % token_start),
                   var_token),
+            201: (re.compile(r'%s{(.*?)}' % token_start),
+                  rvalue_token),
             999: (re.compile(r'.+?(?=(?<!%s)%s(?!%s))|.+'
                              % (token_start, token_start, token_start),
                              re.S),
